==> SyFi-0.2.0.tar.gz: SyFi-0.2.0/syfi/CrouzeixRaviart.cpp <==
#include <CrouzeixRaviart.h> 


CrouzeixRaviart:: CrouzeixRaviart() : StandardFE() {
  order = 1; 
}


int CrouzeixRaviart:: nbf() {
  return Ns.size();  
}

void CrouzeixRaviart:: compute_basis_functions() {

  // see e.g. Brezzi and Fortin book page 116 for the definition
  
  if (p->str() == "ReferenceLine") { 
    cout <<"Can not define the Raviart-Thomas element on a line"<<endl;  
  } else if (p->str() == "ReferenceTriangle" || p->str() == "Triangle"  ) { 
    Triangle& triangle = (Triangle&)(*p); 

    // create the polynomial space
    GiNaC::ex polynom_space = bernstein(1, triangle, "a"); 
    GiNaC::ex polynom = polynom_space.op(0); 
    GiNaC::ex variables = polynom_space.op(1); 
    GiNaC::ex basis = polynom_space.op(2); 

    // create the dofs
    int counter = 0; 
    GiNaC::symbol t("t"); 
    for (int i=1; i<= 3; i++) {
      Line line = triangle.line(i); 
      GiNaC::ex dofi = line.integrate(polynom); 
      dofs.insert(dofs.end(),dofi); 
    }

    // solve the linear system to compute 
    // each of the basis functions
    for (int i=1; i<= 3; i++) {
      GiNaC::lst equations; 
      for (int j=1; j<= 3; j++) {
        equations.append(dofs[j-1] == dirac(i,j));
      }
      GiNaC::ex sub = lsolve(equations, variables); 
      GiNaC::ex Ni = polynom.subs(sub); 
      Ns.insert(Ns.end(),Ni); 
    }

  } else if ( p->str() == "ReferenceTetrahedron" ||  p->str() == "Tetrahedron" ) { 
    Tetrahedron& tetrahedron = (Tetrahedron&)(*p); 
    GiNaC::ex polynom_space = bernstein(1, tetrahedron, "a"); 
    GiNaC::ex polynom = polynom_space.op(0); 
    GiNaC::ex variables = polynom_space.op(1); 
    GiNaC::ex basis = polynom_space.op(2); 


    GiNaC::ex bernstein_pol; 

    int counter = 0; 
    GiNaC::symbol t("t"); 
    // dofs related to edges  
    for (int i=1; i<= 4; i++) {
      Triangle triangle = tetrahedron.triangle(i); 
      GiNaC::ex dofi = triangle.integrate(polynom); 
      dofs.insert(dofs.end(),dofi); 
    }
    for (int i=1; i<= 4; i++) {
      GiNaC::lst equations; 
      for (int j=1; j<= 4; j++) {
        equations.append(dofs[j-1] == dirac(i,j));
      }
      GiNaC::ex sub = lsolve(equations, variables); 
      GiNaC::ex Ni = polynom.subs(sub); 
      Ns.insert(Ns.end(),Ni); 
    }



  }
}


void CrouzeixRaviart:: set(Polygon& p_) {
  StandardFE:: set(p_); 
}


void CrouzeixRaviart:: set(int order_) { 
  if (order_ != 1) {
    cout <<"Only Crouziex-Raviart elements of order 1 is possible"<<endl;  
  }
}


GiNaC::ex CrouzeixRaviart:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex CrouzeixRaviart::N(int i) {  
  return StandardFE::N(i); 
}



// ------------VectorCrouzeixRaviart --- 


VectorCrouzeixRaviart:: VectorCrouzeixRaviart() : StandardFE() {
  order = 1; 
}

int VectorCrouzeixRaviart:: nbf() {
  return StandardFE::nbf();       
}

void VectorCrouzeixRaviart:: compute_basis_functions() {
  CrouzeixRaviart fe; 
  fe.set(*p); 
  fe.compute_basis_functions(); 
  GiNaC::lst zero_list; 
  for (int s=1; s<= size ; s++) {
    zero_list.append(0);  
  }

  for (int i=1; i<= fe.nbf() ; i++) {
    for (int s=1; s<= size ; s++) {
      GiNaC::lst Nis = zero_list;    
      Nis.let_op(s-1) = fe.N(i); 
      GiNaC::ex Nmat = GiNaC::matrix(size,1,Nis); 
      Ns.insert(Ns.end(), Nmat);  

      GiNaC::lst dof = GiNaC::lst(fe.dof(i), s) ; 
      dofs.insert(dofs.end(), dof);  
    }
  }
}


void VectorCrouzeixRaviart:: set_size(int size_) {
  size = size_; 
}


void VectorCrouzeixRaviart:: set(Polygon& p_) {
  StandardFE::set(p_); 
}



void VectorCrouzeixRaviart:: set(int order_) { 
  if (order_ != 1) {
    cout <<"Only Crouziex-Raviart elements of order 1 is possible"<<endl;  
  }
}




GiNaC::ex VectorCrouzeixRaviart:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex VectorCrouzeixRaviart::N(int i) {  
  return StandardFE::N(i); 
}

==> SyFi-0.3.0.tar.gz: SyFi-0.3.0/syfi/CrouzeixRaviart.cpp <==
#include <CrouzeixRaviart.h> 


CrouzeixRaviart:: CrouzeixRaviart() : StandardFE() {
  order = 1; 
}


int CrouzeixRaviart:: nbf() {
  return Ns.size();  
}

void CrouzeixRaviart:: compute_basis_functions() {

  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }
 

  // see e.g. Brezzi and Fortin book page 116 for the definition
  
  if (p->str() == "ReferenceLine") { 
    cout <<"Can not define the Raviart-Thomas element on a line"<<endl;  
  } else if (p->str() == "ReferenceTriangle" || p->str() == "Triangle"  ) { 
    Triangle& triangle = (Triangle&)(*p); 

    // create the polynomial space
    GiNaC::ex polynom_space = bernstein(1, triangle, "a"); 
    GiNaC::ex polynom = polynom_space.op(0); 
    GiNaC::ex variables = polynom_space.op(1); 
    GiNaC::ex basis = polynom_space.op(2); 

    // create the dofs
    int counter = 0; 
    GiNaC::symbol t("t"); 
    for (int i=1; i<= 3; i++) {
      Line line = triangle.line(i); 
      GiNaC::ex dofi = line.integrate(polynom); 
      dofs.insert(dofs.end(),dofi); 
    }

    // solve the linear system to compute 
    // each of the basis functions
    for (int i=1; i<= 3; i++) {
      GiNaC::lst equations; 
      for (int j=1; j<= 3; j++) {
        equations.append(dofs[j-1] == dirac(i,j));
      }
      GiNaC::ex sub = lsolve(equations, variables); 
      GiNaC::ex Ni = polynom.subs(sub); 
      Ns.insert(Ns.end(),Ni); 
    }

  } else if ( p->str() == "ReferenceTetrahedron" ||  p->str() == "Tetrahedron" ) { 
    Tetrahedron& tetrahedron = (Tetrahedron&)(*p); 
    GiNaC::ex polynom_space = bernstein(1, tetrahedron, "a"); 
    GiNaC::ex polynom = polynom_space.op(0); 
    GiNaC::ex variables = polynom_space.op(1); 
    GiNaC::ex basis = polynom_space.op(2); 


    GiNaC::ex bernstein_pol; 

    int counter = 0; 
    GiNaC::symbol t("t"); 
    // dofs related to edges  
    for (int i=1; i<= 4; i++) {
      Triangle triangle = tetrahedron.triangle(i); 
      GiNaC::ex dofi = triangle.integrate(polynom); 
      dofs.insert(dofs.end(),dofi); 
    }
    for (int i=1; i<= 4; i++) {
      GiNaC::lst equations; 
      for (int j=1; j<= 4; j++) {
        equations.append(dofs[j-1] == dirac(i,j));
      }
      GiNaC::ex sub = lsolve(equations, variables); 
      GiNaC::ex Ni = polynom.subs(sub); 
      Ns.insert(Ns.end(),Ni); 
    }



  }
}


void CrouzeixRaviart:: set(Polygon& p_) {
  StandardFE:: set(p_); 
}


void CrouzeixRaviart:: set(int order_) { 
  if (order_ != 1) {
    cout <<"Only Crouziex-Raviart elements of order 1 is possible"<<endl;  
  }
}


GiNaC::ex CrouzeixRaviart:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex CrouzeixRaviart::N(int i) {  
  return StandardFE::N(i); 
}



// ------------VectorCrouzeixRaviart --- 


VectorCrouzeixRaviart:: VectorCrouzeixRaviart() : StandardFE() {
  order = 1; 
}

int VectorCrouzeixRaviart:: nbf() {
  return StandardFE::nbf();       
}

void VectorCrouzeixRaviart:: compute_basis_functions() {
  CrouzeixRaviart fe; 
  fe.set(*p); 
  fe.compute_basis_functions(); 
  GiNaC::lst zero_list; 
  for (int s=1; s<= size ; s++) {
    zero_list.append(0);  
  }

  for (int i=0; i< fe.nbf() ; i++) {
    for (int s=0; s< size ; s++) {
      GiNaC::lst Nis = zero_list;    
      Nis.let_op(s) = fe.N(i); 
      GiNaC::ex Nmat = GiNaC::matrix(size,1,Nis); 
      Ns.insert(Ns.end(), Nmat);  

      GiNaC::lst dof = GiNaC::lst(fe.dof(i), s) ; 
      dofs.insert(dofs.end(), dof);  
    }
  }
}


void VectorCrouzeixRaviart:: set_size(int size_) {
  size = size_; 
}


void VectorCrouzeixRaviart:: set(Polygon& p_) {
  StandardFE::set(p_); 
}



void VectorCrouzeixRaviart:: set(int order_) { 
  if (order_ != 1) {
    cout <<"Only Crouziex-Raviart elements of order 1 is possible"<<endl;  
  }
}




GiNaC::ex VectorCrouzeixRaviart:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex VectorCrouzeixRaviart::N(int i) {  
  return StandardFE::N(i); 
}
